@@ -1,23 +1,22 @@
-    def __del__(self):
+    def __del__(self) -> None:
         try:
-           for cb,opaque in self.domainEventCallbacks.items():
-               del self.domainEventCallbacks[cb]
-           del self.domainEventCallbacks
-           libvirtmod.virConnectDomainEventDeregister(self._o, self)
+            self.domainEventCallbacks.clear()
+            del self.domainEventCallbacks
+            libvirtmod.virConnectDomainEventDeregister(self._o, self)
         except AttributeError:
-           pass
+            pass
 
         if self._o is not None:
             libvirtmod.virConnectClose(self._o)
         self._o = None
 
-    def __enter__(self):
+    def __enter__(self) -> 'virConnect':
         return self
 
-    def __exit__(self, exc_type_, exc_value_, traceback_):
+    def __exit__(self, exc_type_: Optional[Type[BaseException]], exc_value_: Optional[BaseException], traceback_: Optional[TracebackType]) -> None:
         self.close()
 
-    def domainEventDeregister(self, cb):
+    def domainEventDeregister(self, cb: _DomainCB) -> None:
         """Removes a Domain Event Callback. De-registering for a
            domain callback will disable delivery of this event type """
         try:
@@ -25,31 +24,32 @@
             if len(self.domainEventCallbacks) == 0:
                 del self.domainEventCallbacks
                 ret = libvirtmod.virConnectDomainEventDeregister(self._o, self)
-                if ret == -1: raise libvirtError ('virConnectDomainEventDeregister() failed', conn=self)
+                if ret == -1:
+                    raise libvirtError('virConnectDomainEventDeregister() failed')
         except AttributeError:
             pass
 
-    def domainEventRegister(self, cb, opaque):
+    def domainEventRegister(self, cb: _DomainCB, opaque: _T) -> None:
         """Adds a Domain Event Callback. Registering for a domain
            callback will enable delivery of the events """
         try:
             self.domainEventCallbacks[cb] = opaque
         except AttributeError:
-            self.domainEventCallbacks = {cb:opaque}
+            self.domainEventCallbacks = {cb: opaque}  # type: Dict[_DomainCB, _T]
             ret = libvirtmod.virConnectDomainEventRegister(self._o, self)
-            if ret == -1: raise libvirtError ('virConnectDomainEventRegister() failed', conn=self)
+            if ret == -1:
+                raise libvirtError('virConnectDomainEventRegister() failed')
 
-    def _dispatchDomainEventCallbacks(self, dom, event, detail):
+    def _dispatchDomainEventCallbacks(self, dom: 'virDomain', event: int, detail: int) -> None:
         """Dispatches events to python user domain event callbacks
         """
         try:
-            for cb,opaque in self.domainEventCallbacks.items():
+            for cb, opaque in self.domainEventCallbacks.items():
                 cb(self, virDomain(self, _obj=dom), event, detail, opaque)
-            return 0
         except AttributeError:
             pass
 
-    def _dispatchDomainEventLifecycleCallback(self, dom, event, detail, cbData):
+    def _dispatchDomainEventLifecycleCallback(self, dom: 'virDomain', event: int, detail: int, cbData: Dict[str, Any]) -> int:
         """Dispatches events to python user domain lifecycle event callbacks
         """
         cb = cbData["cb"]
@@ -58,7 +58,7 @@
         cb(self, virDomain(self, _obj=dom), event, detail, opaque)
         return 0
 
-    def _dispatchDomainEventGenericCallback(self, dom, cbData):
+    def _dispatchDomainEventGenericCallback(self, dom: 'virDomain', cbData: Dict[str, Any]) -> int:
         """Dispatches events to python user domain generic event callbacks
         """
         cb = cbData["cb"]
@@ -67,16 +67,16 @@
         cb(self, virDomain(self, _obj=dom), opaque)
         return 0
 
-    def _dispatchDomainEventRTCChangeCallback(self, dom, offset, cbData):
+    def _dispatchDomainEventRTCChangeCallback(self, dom: 'virDomain', offset: int, cbData: Dict[str, Any]) -> int:
         """Dispatches events to python user domain RTC change event callbacks
         """
         cb = cbData["cb"]
         opaque = cbData["opaque"]
 
-        cb(self, virDomain(self, _obj=dom), offset ,opaque)
+        cb(self, virDomain(self, _obj=dom), offset, opaque)
         return 0
 
-    def _dispatchDomainEventWatchdogCallback(self, dom, action, cbData):
+    def _dispatchDomainEventWatchdogCallback(self, dom: 'virDomain', action: int, cbData: Dict[str, Any]) -> int:
         """Dispatches events to python user domain watchdog event callbacks
         """
         cb = cbData["cb"]
@@ -85,8 +85,7 @@
         cb(self, virDomain(self, _obj=dom), action, opaque)
         return 0
 
-    def _dispatchDomainEventIOErrorCallback(self, dom, srcPath, devAlias,
-                                            action, cbData):
+    def _dispatchDomainEventIOErrorCallback(self, dom: 'virDomain', srcPath: str, devAlias: str, action: int, cbData: Dict[str, Any]) -> int:
         """Dispatches events to python user domain IO error event callbacks
         """
         cb = cbData["cb"]
@@ -95,9 +94,7 @@
         cb(self, virDomain(self, _obj=dom), srcPath, devAlias, action, opaque)
         return 0
 
-    def _dispatchDomainEventIOErrorReasonCallback(self, dom, srcPath,
-                                                  devAlias, action, reason,
-                                                  cbData):
+    def _dispatchDomainEventIOErrorReasonCallback(self, dom: 'virDomain', srcPath: str, devAlias: str, action: int, reason: int, cbData: Dict[str, Any]) -> int:
         """Dispatches events to python user domain IO error event callbacks
         """
         cb = cbData["cb"]
@@ -107,9 +104,7 @@
            reason, opaque)
         return 0
 
-    def _dispatchDomainEventGraphicsCallback(self, dom, phase, localAddr,
-                                            remoteAddr, authScheme, subject,
-                                            cbData):
+    def _dispatchDomainEventGraphicsCallback(self, dom: 'virDomain', phase: int, localAddr: Any, remoteAddr: Any, authScheme: str, subject: Any, cbData: Dict[str, Any]) -> int:
         """Dispatches events to python user domain graphics event callbacks
         """
         cb = cbData["cb"]
@@ -119,7 +114,7 @@
            authScheme, subject, opaque)
         return 0
 
-    def _dispatchDomainEventBlockJobCallback(self, dom, disk, type, status, cbData):
+    def _dispatchDomainEventBlockJobCallback(self, dom: 'virDomain', disk: str, type: int, status: int, cbData: Dict[str, Any]) -> None:
         """Dispatches events to python user domain blockJob/blockJob2 event callbacks
         """
         try:
@@ -127,11 +122,10 @@
             opaque = cbData["opaque"]
 
             cb(self, virDomain(self, _obj=dom), disk, type, status, opaque)
-            return 0
         except AttributeError:
             pass
 
-    def _dispatchDomainEventDiskChangeCallback(self, dom, oldSrcPath, newSrcPath, devAlias, reason, cbData):
+    def _dispatchDomainEventDiskChangeCallback(self, dom: 'virDomain', oldSrcPath: str, newSrcPath: str, devAlias: str, reason: int, cbData: Dict[str, Any]) -> int:
         """Dispatches event to python user domain diskChange event callbacks
         """
         cb = cbData["cb"]
@@ -140,7 +134,7 @@
         cb(self, virDomain(self, _obj=dom), oldSrcPath, newSrcPath, devAlias, reason, opaque)
         return 0
 
-    def _dispatchDomainEventTrayChangeCallback(self, dom, devAlias, reason, cbData):
+    def _dispatchDomainEventTrayChangeCallback(self, dom: 'virDomain', devAlias: str, reason: int, cbData: Dict[str, Any]) -> int:
         """Dispatches event to python user domain trayChange event callbacks
         """
         cb = cbData["cb"]
@@ -149,7 +143,7 @@
         cb(self, virDomain(self, _obj=dom), devAlias, reason, opaque)
         return 0
 
-    def _dispatchDomainEventPMWakeupCallback(self, dom, reason, cbData):
+    def _dispatchDomainEventPMWakeupCallback(self, dom: 'virDomain', reason: int, cbData: Dict[str, Any]) -> int:
         """Dispatches event to python user domain pmwakeup event callbacks
         """
         cb = cbData["cb"]
@@ -158,7 +152,7 @@
         cb(self, virDomain(self, _obj=dom), reason, opaque)
         return 0
 
-    def _dispatchDomainEventPMSuspendCallback(self, dom, reason, cbData):
+    def _dispatchDomainEventPMSuspendCallback(self, dom: 'virDomain', reason: int, cbData: Dict[str, Any]) -> int:
         """Dispatches event to python user domain pmsuspend event callbacks
         """
         cb = cbData["cb"]
@@ -167,7 +161,7 @@
         cb(self, virDomain(self, _obj=dom), reason, opaque)
         return 0
 
-    def _dispatchDomainEventBalloonChangeCallback(self, dom, actual, cbData):
+    def _dispatchDomainEventBalloonChangeCallback(self, dom: 'virDomain', actual: int, cbData: Dict[str, Any]) -> int:
         """Dispatches events to python user domain balloon change event callbacks
         """
         cb = cbData["cb"]
@@ -176,7 +170,7 @@
         cb(self, virDomain(self, _obj=dom), actual, opaque)
         return 0
 
-    def _dispatchDomainEventPMSuspendDiskCallback(self, dom, reason, cbData):
+    def _dispatchDomainEventPMSuspendDiskCallback(self, dom: 'virDomain', reason: int, cbData: Dict[str, Any]) -> int:
         """Dispatches event to python user domain pmsuspend-disk event callbacks
         """
         cb = cbData["cb"]
@@ -185,7 +179,7 @@
         cb(self, virDomain(self, _obj=dom), reason, opaque)
         return 0
 
-    def _dispatchDomainEventDeviceRemovedCallback(self, dom, devAlias, cbData):
+    def _dispatchDomainEventDeviceRemovedCallback(self, dom: 'virDomain', devAlias: str, cbData: Dict[str, Any]) -> int:
         """Dispatches event to python user domain device removed event callbacks
         """
         cb = cbData["cb"]
@@ -194,7 +188,7 @@
         cb(self, virDomain(self, _obj=dom), devAlias, opaque)
         return 0
 
-    def _dispatchDomainEventTunableCallback(self, dom, params, cbData):
+    def _dispatchDomainEventTunableCallback(self, dom: 'virDomain', params: Any, cbData: Dict[str, Any]) -> int:
         """Dispatches event to python user domain tunable event callbacks
         """
         cb = cbData["cb"]
@@ -203,7 +197,7 @@
         cb(self, virDomain(self, _obj=dom), params, opaque)
         return 0
 
-    def _dispatchDomainEventAgentLifecycleCallback(self, dom, state, reason, cbData):
+    def _dispatchDomainEventAgentLifecycleCallback(self, dom: 'virDomain', state: int, reason: int, cbData: Dict[str, Any]) -> int:
         """Dispatches event to python user domain agent lifecycle event callback
         """
 
@@ -213,7 +207,7 @@
         cb(self, virDomain(self, _obj=dom), state, reason, opaque)
         return 0
 
-    def _dispatchDomainEventDeviceAddedCallback(self, dom, devAlias, cbData):
+    def _dispatchDomainEventDeviceAddedCallback(self, dom: 'virDomain', devAlias: str, cbData: Dict[str, Any]) -> int:
         """Dispatches event to python user domain device added event callbacks
         """
         cb = cbData["cb"]
@@ -222,7 +216,7 @@
         cb(self, virDomain(self, _obj=dom), devAlias, opaque)
         return 0
 
-    def _dispatchDomainEventMigrationIterationCallback(self, dom, iteration, cbData):
+    def _dispatchDomainEventMigrationIterationCallback(self, dom: 'virDomain', iteration: int, cbData: Dict[str, Any]) -> int:
         """Dispatches event to python user domain migration iteration event callbacks
         """
         cb = cbData["cb"]
@@ -231,7 +225,7 @@
         cb(self, virDomain(self, _obj=dom), iteration, opaque)
         return 0
 
-    def _dispatchDomainEventJobCompletedCallback(self, dom, params, cbData):
+    def _dispatchDomainEventJobCompletedCallback(self, dom: 'virDomain', params: Dict[str, Any], cbData: Dict[str, Any]) -> int:
         """Dispatches event to python user domain job completed callbacks
         """
         cb = cbData["cb"]
@@ -240,7 +234,7 @@
         cb(self, virDomain(self, _obj=dom), params, opaque)
         return 0
 
-    def _dispatchDomainEventDeviceRemovalFailedCallback(self, dom, devAlias, cbData):
+    def _dispatchDomainEventDeviceRemovalFailedCallback(self, dom: 'virDomain', devAlias: str, cbData: Dict[str, Any]) -> int:
         """Dispatches event to python user domain device removal failed event callbacks
         """
         cb = cbData["cb"]
@@ -249,7 +243,7 @@
         cb(self, virDomain(self, _obj=dom), devAlias, opaque)
         return 0
 
-    def _dispatchDomainEventMetadataChangeCallback(self, dom, mtype, nsuri, cbData):
+    def _dispatchDomainEventMetadataChangeCallback(self, dom: 'virDomain', mtype: int, nsuri: str, cbData: Dict[str, Any]) -> int:
         """Dispatches event to python user domain metadata change event callbacks
         """
         cb = cbData["cb"]
@@ -258,7 +252,7 @@
         cb(self, virDomain(self, _obj=dom), mtype, nsuri, opaque)
         return 0
 
-    def _dispatchDomainEventBlockThresholdCallback(self, dom, dev, path, threshold, excess, cbData):
+    def _dispatchDomainEventBlockThresholdCallback(self, dom: 'virDomain', dev: str, path: str, threshold: int, excess: int, cbData: Dict[str, Any]) -> int:
         """Dispatches event to python user domain block device threshold event callbacks
         """
         cb = cbData["cb"]
@@ -267,17 +261,18 @@
         cb(self, virDomain(self, _obj=dom), dev, path, threshold, excess, opaque)
         return 0
 
-    def domainEventDeregisterAny(self, callbackID):
+    def domainEventDeregisterAny(self, callbackID: int) -> None:
         """Removes a Domain Event Callback. De-registering for a
            domain callback will disable delivery of this event type """
         try:
             ret = libvirtmod.virConnectDomainEventDeregisterAny(self._o, callbackID)
-            if ret == -1: raise libvirtError ('virConnectDomainEventDeregisterAny() failed', conn=self)
+            if ret == -1:
+                raise libvirtError('virConnectDomainEventDeregisterAny() failed')
             del self.domainEventCallbackID[callbackID]
         except AttributeError:
             pass
 
-    def _dispatchNetworkEventLifecycleCallback(self, net, event, detail, cbData):
+    def _dispatchNetworkEventLifecycleCallback(self, net: 'virNetwork', event: int, detail: int, cbData: Dict[str, Any]) -> int:
         """Dispatches events to python user network lifecycle event callbacks
         """
         cb = cbData["cb"]
@@ -286,47 +281,48 @@
         cb(self, virNetwork(self, _obj=net), event, detail, opaque)
         return 0
 
-    def networkEventDeregisterAny(self, callbackID):
+    def networkEventDeregisterAny(self, callbackID: int) -> None:
         """Removes a Network Event Callback. De-registering for a
            network callback will disable delivery of this event type"""
         try:
             ret = libvirtmod.virConnectNetworkEventDeregisterAny(self._o, callbackID)
-            if ret == -1: raise libvirtError ('virConnectNetworkEventDeregisterAny() failed', conn=self)
+            if ret == -1:
+                raise libvirtError('virConnectNetworkEventDeregisterAny() failed')
             del self.networkEventCallbackID[callbackID]
         except AttributeError:
             pass
 
-    def networkEventRegisterAny(self, net, eventID, cb, opaque):
+    def networkEventRegisterAny(self, net: Optional['virNetwork'], eventID: int, cb: Callable, opaque: _T) -> int:
         """Adds a Network Event Callback. Registering for a network
            callback will enable delivery of the events"""
         if not hasattr(self, 'networkEventCallbackID'):
-            self.networkEventCallbackID = {}
-        cbData = { "cb": cb, "conn": self, "opaque": opaque }
+            self.networkEventCallbackID = {}  # type: Dict[int, _T]
+        cbData = {"cb": cb, "conn": self, "opaque": opaque}
         if net is None:
             ret = libvirtmod.virConnectNetworkEventRegisterAny(self._o, None, eventID, cbData)
         else:
             ret = libvirtmod.virConnectNetworkEventRegisterAny(self._o, net._o, eventID, cbData)
         if ret == -1:
-            raise libvirtError ('virConnectNetworkEventRegisterAny() failed', conn=self)
+            raise libvirtError('virConnectNetworkEventRegisterAny() failed')
         self.networkEventCallbackID[ret] = opaque
         return ret
 
-    def domainEventRegisterAny(self, dom, eventID, cb, opaque):
+    def domainEventRegisterAny(self, dom: Optional['virDomain'], eventID: int, cb: Callable, opaque: _T) -> int:
         """Adds a Domain Event Callback. Registering for a domain
            callback will enable delivery of the events """
         if not hasattr(self, 'domainEventCallbackID'):
-            self.domainEventCallbackID = {}
-        cbData = { "cb": cb, "conn": self, "opaque": opaque }
+            self.domainEventCallbackID = {}  # type: Dict[int, _T]
+        cbData = {"cb": cb, "conn": self, "opaque": opaque}
         if dom is None:
             ret = libvirtmod.virConnectDomainEventRegisterAny(self._o, None, eventID, cbData)
         else:
             ret = libvirtmod.virConnectDomainEventRegisterAny(self._o, dom._o, eventID, cbData)
         if ret == -1:
-            raise libvirtError ('virConnectDomainEventRegisterAny() failed', conn=self)
+            raise libvirtError('virConnectDomainEventRegisterAny() failed')
         self.domainEventCallbackID[ret] = opaque
         return ret
 
-    def _dispatchStoragePoolEventLifecycleCallback(self, pool, event, detail, cbData):
+    def _dispatchStoragePoolEventLifecycleCallback(self, pool: 'virStoragePool', event: int, detail: int, cbData: Dict[str, Any]) -> int:
         """Dispatches events to python user storage pool
            lifecycle event callbacks
         """
@@ -336,7 +332,7 @@
         cb(self, virStoragePool(self, _obj=pool), event, detail, opaque)
         return 0
 
-    def _dispatchStoragePoolEventGenericCallback(self, pool, cbData):
+    def _dispatchStoragePoolEventGenericCallback(self, pool: 'virStoragePool', cbData: Dict[str, Any]) -> int:
         """Dispatches events to python user storage pool
            generic event callbacks
         """
@@ -346,32 +342,33 @@
         cb(self, virStoragePool(self, _obj=pool), opaque)
         return 0
 
-    def storagePoolEventDeregisterAny(self, callbackID):
+    def storagePoolEventDeregisterAny(self, callbackID: int) -> None:
         """Removes a Storage Pool Event Callback. De-registering for a
            storage pool callback will disable delivery of this event type"""
         try:
             ret = libvirtmod.virConnectStoragePoolEventDeregisterAny(self._o, callbackID)
-            if ret == -1: raise libvirtError ('virConnectStoragePoolEventDeregisterAny() failed', conn=self)
+            if ret == -1:
+                raise libvirtError('virConnectStoragePoolEventDeregisterAny() failed')
             del self.storagePoolEventCallbackID[callbackID]
         except AttributeError:
             pass
 
-    def storagePoolEventRegisterAny(self, pool, eventID, cb, opaque):
+    def storagePoolEventRegisterAny(self, pool: Optional['virStoragePool'], eventID: int, cb: Callable, opaque: _T) -> int:
         """Adds a Storage Pool Event Callback. Registering for a storage pool
            callback will enable delivery of the events"""
         if not hasattr(self, 'storagePoolEventCallbackID'):
-            self.storagePoolEventCallbackID = {}
-        cbData = { "cb": cb, "conn": self, "opaque": opaque }
+            self.storagePoolEventCallbackID = {}  # type: Dict[int, _T]
+        cbData = {"cb": cb, "conn": self, "opaque": opaque}
         if pool is None:
             ret = libvirtmod.virConnectStoragePoolEventRegisterAny(self._o, None, eventID, cbData)
         else:
             ret = libvirtmod.virConnectStoragePoolEventRegisterAny(self._o, pool._o, eventID, cbData)
         if ret == -1:
-            raise libvirtError ('virConnectStoragePoolEventRegisterAny() failed', conn=self)
+            raise libvirtError('virConnectStoragePoolEventRegisterAny() failed')
         self.storagePoolEventCallbackID[ret] = opaque
         return ret
 
-    def _dispatchNodeDeviceEventLifecycleCallback(self, dev, event, detail, cbData):
+    def _dispatchNodeDeviceEventLifecycleCallback(self, dev: 'virNodeDevice', event: int, detail: int, cbData: Dict[str, Any]) -> int:
         """Dispatches events to python user node device
            lifecycle event callbacks
         """
@@ -381,7 +378,7 @@
         cb(self, virNodeDevice(self, _obj=dev), event, detail, opaque)
         return 0
 
-    def _dispatchNodeDeviceEventGenericCallback(self, dev, cbData):
+    def _dispatchNodeDeviceEventGenericCallback(self, dev: 'virNodeDevice', cbData: Dict[str, Any]) -> int:
         """Dispatches events to python user node device
            generic event callbacks
         """
@@ -391,32 +388,33 @@
         cb(self, virNodeDevice(self, _obj=dev), opaque)
         return 0
 
-    def nodeDeviceEventDeregisterAny(self, callbackID):
+    def nodeDeviceEventDeregisterAny(self, callbackID: int) -> None:
         """Removes a Node Device Event Callback. De-registering for a
            node device callback will disable delivery of this event type"""
         try:
             ret = libvirtmod.virConnectNodeDeviceEventDeregisterAny(self._o, callbackID)
-            if ret == -1: raise libvirtError ('virConnectNodeDeviceEventDeregisterAny() failed', conn=self)
+            if ret == -1:
+                raise libvirtError('virConnectNodeDeviceEventDeregisterAny() failed')
             del self.nodeDeviceEventCallbackID[callbackID]
         except AttributeError:
             pass
 
-    def nodeDeviceEventRegisterAny(self, dev, eventID, cb, opaque):
+    def nodeDeviceEventRegisterAny(self, dev: Optional['virNodeDevice'], eventID: int, cb: Callable, opaque: _T) -> int:
         """Adds a Node Device Event Callback. Registering for a node device
            callback will enable delivery of the events"""
         if not hasattr(self, 'nodeDeviceEventCallbackID'):
-            self.nodeDeviceEventCallbackID = {}
-        cbData = { "cb": cb, "conn": self, "opaque": opaque }
+            self.nodeDeviceEventCallbackID = {}  # type: Dict[int, _T]
+        cbData = {"cb": cb, "conn": self, "opaque": opaque}
         if dev is None:
             ret = libvirtmod.virConnectNodeDeviceEventRegisterAny(self._o, None, eventID, cbData)
         else:
             ret = libvirtmod.virConnectNodeDeviceEventRegisterAny(self._o, dev._o, eventID, cbData)
         if ret == -1:
-            raise libvirtError ('virConnectNodeDeviceEventRegisterAny() failed', conn=self)
+            raise libvirtError('virConnectNodeDeviceEventRegisterAny() failed')
         self.nodeDeviceEventCallbackID[ret] = opaque
         return ret
 
-    def _dispatchSecretEventLifecycleCallback(self, secret, event, detail, cbData):
+    def _dispatchSecretEventLifecycleCallback(self, secret: 'virSecret', event: int, detail: int, cbData: Dict[str, Any]) -> int:
         """Dispatches events to python user secret lifecycle event callbacks
         """
         cb = cbData["cb"]
@@ -425,7 +423,7 @@
         cb(self, virSecret(self, _obj=secret), event, detail, opaque)
         return 0
 
-    def _dispatchSecretEventGenericCallback(self, secret, cbData):
+    def _dispatchSecretEventGenericCallback(self, secret: 'virSecret', cbData: Dict[str, Any]) -> int:
         """Dispatches events to python user secret generic event callbacks
         """
         cb = cbData["cb"]
@@ -434,128 +432,97 @@
         cb(self, virSecret(self, _obj=secret), opaque)
         return 0
 
-    def secretEventDeregisterAny(self, callbackID):
+    def secretEventDeregisterAny(self, callbackID: int) -> None:
         """Removes a Secret Event Callback. De-registering for a
            secret callback will disable delivery of this event type"""
         try:
             ret = libvirtmod.virConnectSecretEventDeregisterAny(self._o, callbackID)
-            if ret == -1: raise libvirtError ('virConnectSecretEventDeregisterAny() failed', conn=self)
+            if ret == -1:
+                raise libvirtError('virConnectSecretEventDeregisterAny() failed')
             del self.secretEventCallbackID[callbackID]
         except AttributeError:
             pass
 
-    def secretEventRegisterAny(self, secret, eventID, cb, opaque):
+    def secretEventRegisterAny(self, secret: Optional['virSecret'], eventID: int, cb: Callable, opaque: _T) -> int:
         """Adds a Secret Event Callback. Registering for a secret
            callback will enable delivery of the events"""
         if not hasattr(self, 'secretEventCallbackID'):
-            self.secretEventCallbackID = {}
-        cbData = { "cb": cb, "conn": self, "opaque": opaque }
+            self.secretEventCallbackID = {}  # type: Dict[int, _T]
+        cbData = {"cb": cb, "conn": self, "opaque": opaque}
         if secret is None:
             ret = libvirtmod.virConnectSecretEventRegisterAny(self._o, None, eventID, cbData)
         else:
             ret = libvirtmod.virConnectSecretEventRegisterAny(self._o, secret._o, eventID, cbData)
         if ret == -1:
-            raise libvirtError ('virConnectSecretEventRegisterAny() failed', conn=self)
+            raise libvirtError('virConnectSecretEventRegisterAny() failed')
         self.secretEventCallbackID[ret] = opaque
         return ret
 
-    def listAllDomains(self, flags=0):
+    def listAllDomains(self, flags: int = 0) -> List['virDomain']:
         """List all domains and returns a list of domain objects"""
         ret = libvirtmod.virConnectListAllDomains(self._o, flags)
         if ret is None:
-            raise libvirtError("virConnectListAllDomains() failed", conn=self)
+            raise libvirtError("virConnectListAllDomains() failed")
 
-        retlist = list()
-        for domptr in ret:
-            retlist.append(virDomain(self, _obj=domptr))
+        return [virDomain(self, _obj=domptr) for domptr in ret]
 
-        return retlist
-
-    def listAllStoragePools(self, flags=0):
+    def listAllStoragePools(self, flags: int = 0) -> List['virStoragePool']:
         """Returns a list of storage pool objects"""
         ret = libvirtmod.virConnectListAllStoragePools(self._o, flags)
         if ret is None:
-            raise libvirtError("virConnectListAllStoragePools() failed", conn=self)
+            raise libvirtError("virConnectListAllStoragePools() failed")
 
-        retlist = list()
-        for poolptr in ret:
-            retlist.append(virStoragePool(self, _obj=poolptr))
+        return [virStoragePool(self, _obj=poolptr) for poolptr in ret]
 
-        return retlist
-
-    def listAllNetworks(self, flags=0):
+    def listAllNetworks(self, flags: int = 0) -> List['virNetwork']:
         """Returns a list of network objects"""
         ret = libvirtmod.virConnectListAllNetworks(self._o, flags)
         if ret is None:
-            raise libvirtError("virConnectListAllNetworks() failed", conn=self)
+            raise libvirtError("virConnectListAllNetworks() failed")
 
-        retlist = list()
-        for netptr in ret:
-            retlist.append(virNetwork(self, _obj=netptr))
+        return [virNetwork(self, _obj=netptr) for netptr in ret]
 
-        return retlist
-
-    def listAllInterfaces(self, flags=0):
+    def listAllInterfaces(self, flags: int = 0) -> List['virInterface']:
         """Returns a list of interface objects"""
         ret = libvirtmod.virConnectListAllInterfaces(self._o, flags)
         if ret is None:
-            raise libvirtError("virConnectListAllInterfaces() failed", conn=self)
+            raise libvirtError("virConnectListAllInterfaces() failed")
 
-        retlist = list()
-        for ifaceptr in ret:
-            retlist.append(virInterface(self, _obj=ifaceptr))
+        return [virInterface(self, _obj=ifaceptr) for ifaceptr in ret]
 
-        return retlist
-
-    def listAllDevices(self, flags=0):
+    def listAllDevices(self, flags: int = 0) -> List['virNodeDevice']:
         """Returns a list of host node device objects"""
         ret = libvirtmod.virConnectListAllNodeDevices(self._o, flags)
         if ret is None:
-            raise libvirtError("virConnectListAllNodeDevices() failed", conn=self)
+            raise libvirtError("virConnectListAllNodeDevices() failed")
 
-        retlist = list()
-        for devptr in ret:
-            retlist.append(virNodeDevice(self, _obj=devptr))
+        return [virNodeDevice(self, _obj=devptr) for devptr in ret]
 
-        return retlist
-
-    def listAllNWFilters(self, flags=0):
+    def listAllNWFilters(self, flags: int = 0) -> List['virNWFilter']:
         """Returns a list of network filter objects"""
         ret = libvirtmod.virConnectListAllNWFilters(self._o, flags)
         if ret is None:
-            raise libvirtError("virConnectListAllNWFilters() failed", conn=self)
+            raise libvirtError("virConnectListAllNWFilters() failed")
 
-        retlist = list()
-        for filter_ptr in ret:
-            retlist.append(virNWFilter(self, _obj=filter_ptr))
+        return [virNWFilter(self, _obj=filter_ptr) for filter_ptr in ret]
 
-        return retlist
-
-    def listAllNWFilterBindings(self, flags=0):
+    def listAllNWFilterBindings(self, flags: int = 0) -> List['virNWFilterBinding']:
         """Returns a list of network filter binding objects"""
         ret = libvirtmod.virConnectListAllNWFilterBindings(self._o, flags)
         if ret is None:
-            raise libvirtError("virConnectListAllNWFilterBindings() failed", conn=self)
+            raise libvirtError("virConnectListAllNWFilterBindings() failed")
 
-        retlist = list()
-        for filter_ptr in ret:
-            retlist.append(virNWFilterBinding(self, _obj=filter_ptr))
+        return [virNWFilterBinding(self, _obj=filter_ptr) for filter_ptr in ret]
 
-        return retlist
-
-    def listAllSecrets(self, flags=0):
+    def listAllSecrets(self, flags: int = 0) -> List['virSecret']:
         """Returns a list of secret objects"""
         ret = libvirtmod.virConnectListAllSecrets(self._o, flags)
         if ret is None:
-            raise libvirtError("virConnectListAllSecrets() failed", conn=self)
+            raise libvirtError("virConnectListAllSecrets() failed")
 
-        retlist = list()
-        for secret_ptr in ret:
-            retlist.append(virSecret(self, _obj=secret_ptr))
+        return [virSecret(self, _obj=secret_ptr) for secret_ptr in ret]
 
-        return retlist
-
-    def _dispatchCloseCallback(self, reason, cbData):
+    def _dispatchCloseCallback(self, reason: int, cbData: Dict[str, Any]) -> int:
         """Dispatches events to python user close callback"""
         cb = cbData["cb"]
         opaque = cbData["opaque"]
@@ -563,22 +530,22 @@
         cb(self, reason, opaque)
         return 0
 
-
-    def unregisterCloseCallback(self):
+    def unregisterCloseCallback(self) -> None:
         """Removes a close event callback"""
         ret = libvirtmod.virConnectUnregisterCloseCallback(self._o)
-        if ret == -1: raise libvirtError ('virConnectUnregisterCloseCallback() failed', conn=self)
+        if ret == -1:
+            raise libvirtError('virConnectUnregisterCloseCallback() failed')
 
-    def registerCloseCallback(self, cb, opaque):
+    def registerCloseCallback(self, cb: Callable, opaque: _T) -> int:
         """Adds a close event callback, providing a notification
          when a connection fails / closes"""
-        cbData = { "cb": cb, "conn": self, "opaque": opaque }
+        cbData = {"cb": cb, "conn": self, "opaque": opaque}
         ret = libvirtmod.virConnectRegisterCloseCallback(self._o, cbData)
         if ret == -1:
-            raise libvirtError ('virConnectRegisterCloseCallback() failed', conn=self)
+            raise libvirtError('virConnectRegisterCloseCallback() failed')
         return ret
 
-    def createXMLWithFiles(self, xmlDesc, files, flags=0):
+    def createXMLWithFiles(self, xmlDesc: str, files: List[int], flags: int = 0) -> 'virDomain':
         """Launch a new guest domain, based on an XML description similar
         to the one returned by virDomainGetXMLDesc()
         This function may require privileged access to the hypervisor.
@@ -604,11 +571,12 @@
         libvirtd daemon. Any domains marked for auto destroy will
         block attempts at migration, save-to-file, or snapshots. """
         ret = libvirtmod.virDomainCreateXMLWithFiles(self._o, xmlDesc, files, flags)
-        if ret is None:raise libvirtError('virDomainCreateXMLWithFiles() failed', conn=self)
-        __tmp = virDomain(self,_obj=ret)
+        if ret is None:
+            raise libvirtError('virDomainCreateXMLWithFiles() failed')
+        __tmp = virDomain(self, _obj=ret)
         return __tmp
 
-    def getAllDomainStats(self, stats = 0, flags=0):
+    def getAllDomainStats(self, stats: int = 0, flags: int = 0) -> List[Tuple['virDomain', Dict[str, Any]]]:
         """Query statistics for all domains on a given connection.
 
         Report statistics of various parameters for a running VM according to @stats
@@ -652,16 +620,11 @@
         VIR_CONNECT_GET_ALL_DOMAINS_STATS_OTHER for all other states. """
         ret = libvirtmod.virConnectGetAllDomainStats(self._o, stats, flags)
         if ret is None:
-            raise libvirtError("virConnectGetAllDomainStats() failed", conn=self)
+            raise libvirtError("virConnectGetAllDomainStats() failed")
 
-        retlist = list()
-        for elem in ret:
-            record = (virDomain(self, _obj=elem[0]) , elem[1])
-            retlist.append(record)
+        return [(virDomain(self, _obj=elem[0]), elem[1]) for elem in ret]
 
-        return retlist
-
-    def domainListGetStats(self, doms, stats=0, flags=0):
+    def domainListGetStats(self, doms: List['virDomain'], stats: int = 0, flags: int = 0) -> List[Tuple['virDomain', Dict[str, Any]]]:
         """ Query statistics for given domains.
 
         Report statistics of various parameters for a running VM according to @stats
@@ -693,17 +656,12 @@
         domlist = list()
         for dom in doms:
             if not isinstance(dom, virDomain):
-                raise libvirtError("domain list contains non-domain elements", conn=self)
+                raise libvirtError("domain list contains non-domain elements")
 
             domlist.append(dom._o)
 
         ret = libvirtmod.virDomainListGetStats(self._o, domlist, stats, flags)
         if ret is None:
-            raise libvirtError("virDomainListGetStats() failed", conn=self)
+            raise libvirtError("virDomainListGetStats() failed")
 
-        retlist = list()
-        for elem in ret:
-            record = (virDomain(self, _obj=elem[0]) , elem[1])
-            retlist.append(record)
-
-        return retlist
+        return [(virDomain(self, _obj=elem[0]), elem[1]) for elem in ret]
